//Revised code to display a QuickTime movie in a window under Java 1.4.x and QuickTime 6.4, aka QTJ 6.1
//Version 1.0 beta 2, August 15, 2004 by Jeff Hardin, Dept. of Zoology, Univ. of Wisconsin
//Questions? Contact the author at jdhardin@wisc.edu

//Adapted from basic code courtesy of Chris Adamson, c/o the O'Reilly Network
//Window resize detection was broken under Mac OS X 10.3, JavaVM 1.4.1, but now seems to work under the newest 1.4.2 update!!


import ij.*;
import ij.plugin.PlugIn;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.io.IOException;

import quicktime.*;
import quicktime.util.*;
import quicktime.io.*;
import quicktime.std.movies.*;
import quicktime.std.movies.media.*;
import quicktime.app.view.*;


public class Movie_Player_rev implements PlugIn,WindowListener,KeyListener,ComponentListener {

public Frame f2;
public Movie m;
public Component c;
public MovieController mc;

    public void run (String arg) {
        try {
	String mySystemString = System.getProperty("java.version");
	double myQTJVersion = (double) QTBuild.getVersion() +  (double) QTBuild.getSubVersion()/10;
 	//IJ.log("Java VM version: " + mySystemString + "  QT Java version: " + QTBuild.getVersion() + "." + QTBuild.getSubVersion());
	//IJ.log("Offset to version number: " + mySystemString.indexOf("1.4");
	if  ( ((mySystemString.indexOf("1.4") < 0) && (mySystemString.indexOf("1.5") < 0) && (mySystemString.indexOf("1.6") < 0) && (mySystemString.indexOf("1.7") < 0)) || (myQTJVersion < 6.1))
	{IJ.showMessage("This plugin requires at least Java VM 1.4 and QT Java 6.1.");
	return;}
            QTSession.open();
            QTFile f = QTFile.standardGetFilePreview (new int[0]);
            OpenMovieFile omf = OpenMovieFile.asRead (f);
            m = Movie.fromFile (omf);
           mc = new MovieController (m);
            mc.setVisible(true);
           //need the next line to send key events to the movie controller so we can simulate the arrow key functions Apple broke
            mc.setKeysEnabled(true);
            c = QTFactory.makeQTComponent (mc).asComponent();
            c.setName("Controller");
            f2 = new Frame();
	  f2.setTitle(f.getName());
            f2.addWindowListener(this);            
            f2.addKeyListener(this);
	//Component resize is broken under VM 1.4.1 and Panther, so let's not add the listener!
           f2.addComponentListener(this);
            f2.setName("Window");
            BorderLayout myLayout = new BorderLayout();

            //Under Mac OS X and Java 1.4, add a little space at bottom so that 
            //drag box doesn't overlap with right arrow button in movie controller. Way to go, Apple! :-(

            f2.setLayout(myLayout);
            f2.add(c,"Center");
            Panel myPanel = new Panel();
            f2.add(myPanel,"South");
            f2.pack();
            //Under Mac OS X, resize events aren't properly registered, so turn off resizing for now. Arghhh!!!!
           f2.setResizable(true);
           f2.setVisible(true);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
		     public void windowClosing (WindowEvent e) {
				QTSession.close();
				f2.setVisible(false);
				f2.dispose();
			}			
		     public void windowClosed (WindowEvent e) { 
			}
                        public void windowOpened (WindowEvent e) { 
			}
                        public void windowActivated (WindowEvent e) { 
			}
                        public void windowDeactivated (WindowEvent e) { 
			}
                        public void windowIconified (WindowEvent e) { 
			}
                        public void windowDeiconified (WindowEvent e) { 
			}


                    public void keyPressed (KeyEvent e) 
                    {                        try
                        {
		//need to pass MovieController.key() an int corresponding to standard Java key code; initialize it here
                    int myInt = 0;
                            switch (e.getKeyCode()) 
                            {
                                case KeyEvent.VK_SPACE:
			//play the movie forward by setting rate to +1
                                    if (m.getRate() != 0) 
                                        m.setRate (0);
                                    else
                                        m.setRate (1);
                                    break;
                                case KeyEvent.VK_LEFT:
                                //now we send a MovieController.key() event to the controller
				myInt = KeyEvent.VK_LEFT;
                                    mc.key(myInt,e.getModifiers());                                  
                                    break;
                                case KeyEvent.VK_RIGHT:
                                //now we send a MovieController.key() event to the controller
				myInt = KeyEvent.VK_RIGHT;
                                     mc.key (myInt,e.getModifiers());
                                     break;
                                default:
                                    break;
                            }
                        } catch (QTException ee) {
                            throw new QTRuntimeException (ee);
                        }
}
                    public void keyReleased (KeyEvent e) {}
                    public void keyTyped (KeyEvent e) {}
                    
                    public void componentResized (ComponentEvent e) {
                    //Seems to be broken for now under Mac OS X and Java 1.4.1 VM; only gets increases in size, but not decreases!
					//IJ.log("Component: " + e.getComponent().getName() + "  New size: " + e.getComponent().getSize().getWidth() + "," + e.getComponent().getSize().getHeight());                  
 }
                    public void componentHidden (ComponentEvent e) {}
                    public void componentMoved (ComponentEvent e) {}
                    public void componentShown (ComponentEvent e) {}
                    


}






